/** SALSA/World Wide Computer Project
 *  ObjectHandler recieves incoming objects
 *  and determines if they are actors or messages.
 *  upon recieving actors, it will start them up.
 *  messages get sent to the appropriate actor.
 *
 *  author Travis Desell
 */
package io.core.messaging;

import java.net.*;
import java.io.*;

import salsa.language.ActorState;
import salsa.language.Message;
import salsa.language.RunTime;
import salsa.language.ServiceFactory;
import salsa.messaging.*;

import wwc.messaging.PersistentHandler;
import wwc.messaging.RMSPAcknowledgement;

import io.core.protocol.P2PPacket;

public class PersistentIOHandler extends PersistentHandler implements ReceptionService {
	public PersistentIOHandler() {
	}

	public void process(Socket incoming) {
		// Read in the object.
		Object received = null;
		ObjectInputStream stream = null;
		try {
			stream = new ObjectInputStream( incoming.getInputStream() );
		} catch (IOException e) {
			System.err.println( "IOException occured reading object " + e.getMessage() );
			return;
		}

		while (incoming.isConnected()) {
			try {
				received = stream.readObject();
			} catch (ClassNotFoundException e) {
				System.err.println( "Could not load class for " + received.getClass().getName() );
				System.err.println( "  Is it in the theater CLASSPATH" );
				System.err.println( e.getMessage() );
				break;
			} catch (IOException e) {
				System.err.println( "IOException occured reading object " + e.getMessage() );
				break;
			}

                        if (received instanceof P2PPacket) {
                                ((IOTheater)ServiceFactory.getTheater()).getProtocolHandler().process( (P2PPacket)received);
				break;
 			} else if (received instanceof ActorState) {
				ActorState incomingActor = (ActorState) received;
				processActor(incomingActor, incoming);
				break;
			} else if (received instanceof Message) {  // just serializing data
				Message message = (Message) received;
				processMessage(message);
			} else if (received instanceof TheaterLocator) {
				((IOTheater)ServiceFactory.getTheater()).getNeighbors().add(received);
				break;
			}
		}

		// Close the connection.
		try {
			incoming.close();
		} catch (IOException e) {
			System.err.println( "IOException occured closing connection" + e.getMessage() );
			return;
		}
	}
}
