/**
	RandomActorState:
	RandomActorState gathers information along with RandomActor
	about the actor's surroundings.  It takes care of finding out about
	an actors processing speed.


	Author: Travis Desell
	Last Modified: 9/19/2002
*/

package io.random.language;

import java.io.*;
import java.net.*;
import salsa.naming.*;
import salsa.naming.exceptions.*;
import salsa.language.*;
import salsa.language.exceptions.*;
import io.core.language.IOActorState;
import io.core.messaging.TheaterLocator;
import io.core.messaging.IOTheater;

public class RandomActorState extends ActorState implements IOActorState {
	protected int messagesRecieved = 0;
	protected int messagesProcessed = 0;
	protected boolean scheduled = false;

	public RandomActorState() {
		super();
	}

	public int getRecieved() {
		return messagesRecieved;
	}

	public int getProcessed() {
		return messagesProcessed;
	}

        public void prioritySend(Message message) {
                scheduled = true;
		messagesRecieved++;
                super.prioritySend(message);
        }

        public void send(Message message) {
                messagesRecieved++;
                super.send(message);
        }

	public void migrate(String ualString) {
		//set scheduled to false so when it reaches the other theater
		//it may migrate from there
		reset();
		scheduled = false;

		super.migrate(ualString);
	}

	public synchronized void process(Message message) {
		super.process(message);

		messagesProcessed++;
	}

	public void reset() {
		messagesProcessed = 0;
		messagesRecieved = 0;
	}       
                
	public boolean scheduled() {
		return scheduled;
	}

	public boolean satisfied() {
		//we subtract one from messages recieved in case the actor is processing
		//messages as quickly as it is recieving them, and is in the middle of processing
		//a message
                        
		if (mailbox.isEmpty()) {
			if (messagesProcessed == 0) return true;
			else if (messagesProcessed > messagesRecieved) return true;
			else return false;
		} else {
			if (messagesProcessed > messagesRecieved) {
				return true;
			} else {
				return false;
			}
		}
        }

        public double satisfaction() {
                if (messagesProcessed > 0) {
                        return (messagesRecieved - messagesProcessed) / messagesProcessed;
                } else {
                        return (messagesRecieved * 2);
                }
        }
}
