/** SALSA/World Wide Computer Project
 *  ObjectHandler recieves incoming objects
 *  and determines if they are actors or messages.
 *  upon recieving actors, it will start them up.
 *  messages get sent to the appropriate actor.
 *
 *  author Travis Desell
 */
package io.messaging;

import java.net.*;
import java.io.*;
import java.util.Hashtable;
import java.util.Vector;
import salsa.language.*;
import io.protocol.*;
	
public class ProtocolHandler extends Thread {
	private P2PTheater theater;
	private Object recieved;

	//max time to live should probably be user definable, or 
	//a commandline argument at runtime
	private int maxTimeToLive = 5;


	public ProtocolHandler(P2PTheater _theater, Object _recieved) {
		theater = _theater;
		recieved = _recieved;

		setName( "Protocol Handler Thread" );
	}

	public void run() {
		DecisionAgent decider = theater.getDecisionAgent();

		if (recieved instanceof Ping) {
			//create a pong packet, send it back to who send the ping

			Pong pong = new Pong( (Ping)recieved , theater.getLocation());
			pong.send( ((Ping)recieved).getOrigin() );

		} else if (recieved instanceof Pong) {
			//send it to our decider to see if it should be added to our list of peers,
			//we give it the theater, and the amount of time for a round trip ping/pong

			Pong pong = (Pong)recieved;

			

			theater.getNeighbors().addTheater(pong.getOrigin(), System.currentTimeMillis() - pong.timestamp);

		} else if (recieved instanceof ResourceResponse) {
			MigrationAccept migrationAccept = (MigrationAccept)recieved;

			//forward the information about the remote theaters resources to the decider, who will use
			//this information to decide if it will move actors away
			decider.recievedResponseData(recieved.getOrigin(), recieved.getTheaterData());

		} else if (recieved instanceof ResourceRequest) {
			ResourceRequest ResourceRequest = (ResourceRequest)recieved;

			//if we were the source of the packet, do not do anything.
			if (!reBroadcast(migrationRequest.getOrigin())) {

				if (!decider.overloaded()) {
					ResourceResponse resourceResponse = new ResourceResponse(theater.getLocation(), decider.getTheaterData()); 
					resourceResponse.send(resourceRequest.getOrigin());
					decider.responding();
				}

				if (resourceRequest.timeToLive > maxTimeToLive) {
					resourceRequest.timeToLive = maxTimeToLive;
				}

				resourceRequest.timeToLive--;
				if (resourceRequest.timeToLive > 0) {
					resourceRequest.broadcast(theater.getNeighbors());
				}
			}

		} else if ((recieved instanceof Accept)||(recieved instanceof Reject)) {
			//Either we have found an theater who will send us an actor or we were denied, 
			//however we can now go back to sending more ResourceResponses

			decider.respondingFinished();

		} else if (recieved instanceof NewResource) {
			//found a new available actor:
			//we can add it to our list of actors, if its statistics are good enough

			NewResource newResource = (NewResource)recieved;

			if ( !reBroadcast(newResource.getOrigin()) ) {
				if (!decider.hasTheater(newResource.getOrigin())) {
					Ping ping = new Ping( theater.getLocation());
					ping.send(available.origin);

					decider.addTheater(available.origin);
				}

				if (newResource.timeToLive > maxTimeToLive) {
					newResource.timeToLive = maxTimeToLive;
				}

				newResource.timeToLive--;
				if (newResource.timeToLive > 0) {
					newResource.broadcast(theater.getNeighbors());
				}
			}

		} else if (recieved instanceof TheaterLocator) {
			//We got a peer to add to our list:
			//	ping it, so we can see if we want it
			TheaterLocator theaterLocation = (TheaterLocation)recieved;

			if (!decider.hasTheater(theaterLocation)) {
				decider.addTheater(theaterLocation);

				Ping ping = new Ping( theater.getLocation() );
				ping.send(theaterLocation);
			}
		}
   	}

	private boolean reBroadcast(TheaterLocator packetOrigin) {
		if (!packetOrigin.equals(theater.getLocation)) {
			boolean duplicate = false;

			Vector incomingTheaterQueue = decider.getIncomingTheaterQueue();

			for (int i = 0; i < incomingTheaterQueue.size(); i++) {
				TheaterLocator current = ((TheaterLocator)incomingTheaterQueue.get(i));
				if ( current.equals(packetOrigin)) {
					if (current.getMessageId() >= packetOriginetMessageId()) return true;
					else current = new TheaterLocator(current, packetOrigin.getMessageId());
					duplicate = true;
				}
			}

			if (!duplicate) {
				if (incomingTheaterQueue.size() >= decider.getMaxTheaterQueueSize()) incomingTheaterQueue.remove(0);
				incomingTheaterQueue.add(packetOrigin);
			}
			return false;
		} else {
			return true;
		}
	}
}

